function [ArPort] = ArPortMPs( uMat,h,p,test )
%/*----------------------------------------------------------------------- 
%**  ArPortMPs 
%** 
%**  Purpose:   Portmanteau test of autocorrelation. 
%** 
%** 
%** 
%**  Usage:     ArPort = ArPortMPs( umat,h,p ); 
%** 
%**  Input:     uMat -  TxK matrix of residuals 
%**             h       scalar, test up to hth order serial corrlation 
%**             p       scalar, lags in VAR system behind uMat 
%** 
%**  Output:    ArPort is a 3x1 vector pf Test value|p-value|df 
%** 
%** 
%**  Reference: Lutkepohl (1993), Introduction to Multiple Time Series 
%**             Analysis, 2ed, p. 150. 
%** 
%** 
%** 
%**  Paul Soderlind (Paul.Soderlind@hhs.se), 1995 
%-------------------------------------------------------------------------*/ 

if h < p
    error('fatal (ArPortMPs) h is less than p')
end
[n,k]=size(uMat);
if n < k
    uMat=uMat';
end
[n,k]=size(uMat);

CovU = cov( uMat );

Ph = 0;
c0_1 = inv(CovU);

for i = 1:h ;%loop over lags;
    ci = zeros(k,k);
    for t = i+1:n
        ci=ci+uMat(t-i,:)'*uMat(t,:);
    end
    ci=ci/n;

    Ph = Ph + n*sum(  diag( ci'*c0_1*ci*c0_1 )  );
end

Ardf = k^2 *( h-p );               % - k*cols(z);

if test == 1
    Phpval = 1-chi2cdf( Ph,Ardf );    
    ArPort = [Ph Phpval Ardf]';
else
    ArPort = Ph;
end
