function Fisher =   chainweight(q,p)

%   Fisher =   chainweight(q,p)
%
%   Returns the ideal Fisher price index
%
%   Inputs:
%   q(TxN): matrix containing T observations on quantities for N series
%   p(TxN): matrix containing T observations on prices for N series
%
%   Ouptut:
%   Fisher: Fisher price index

Laspeyres   =   sum(p(2:end,:).*q(1:end-1,:),2)./sum(p(1:end-1,:).*q(1:end-1,:),2);
Paasche     =   sum(p(2:end,:).*q(2:end,:),2)./sum(p(1:end-1,:).*q(2:end,:),2);
Fisher      =   [1;cumprod((Laspeyres.*Paasche).^.5)];