clear all
close all
addpath c:\research\ebc % for filter files
addpath c:\matlab_code\utilities

table1_1    =   xlsread('beadata','table1_1');
table1_2    =   xlsread('beadata','table1_2');
table7_1    =   xlsread('beadata','table7_1');
table3_7    =   xlsread('beadata','table3_7');
table7_11   =   xlsread('beadata','table7_11');
lmanufdur   =   xlsread('empldatasec','manufdur');
lmanufdur   =   lmanufdur(:,14:end);
lmanufdur   =   reshape(lmanufdur',56*4,1);
lmanufdur   =   lmanufdur(1:end-2);
lmanufndur  =   xlsread('empldatasec','manufndur');
lmanufndur  =   lmanufndur(:,14:end);
lmanufndur  =   reshape(lmanufndur',56*4,1);
lmanufndur  =   lmanufndur(1:end-2);
lgovt       =   xlsread('empldatasec','govt');
lgovt       =   lgovt(:,14:end);
lgovt       =   reshape(lgovt',56*4,1);
lgovt       =   lgovt(1:end-2);
lserv       =   xlsread('empldatasec','serv');
lserv       =   lserv(:,14:end);
lserv       =   reshape(lserv',56*4,1);
lserv       =   lserv(1:end-2);
linv        =   lmanufdur(2:end);
lcons       =   lmanufndur(2:end)+lserv(2:end)+lgovt(2:end);

% Real quantities and prices are measured as in Fisher (1999) page 38

% Private sector

GDP     =   table1_2(1,:);
DurN    =   table1_1(3,6:end);
NondurN =   table1_1(4,6:end);
ServN   =   table1_1(5,6:end);
GPDIN   =   table1_1(6,6:end);
QDur    =   table7_1(12,:);
PDur    =   table7_1(13,:);
QNondur =   table7_1(17,:);
PNondur =   table7_1(18,:);
QServ   =   table7_1(22,:);
PServ   =   table7_1(23,:);
QG      =   table7_1(87,:);
PG      =   table7_1(88,:);
LCons   =   (PNondur(2:end).*QNondur(1:end-1)+PServ(2:end).*QServ(1:end-1)+PG(2:end).*QG(1:end-1))...
    ./(PNondur(1:end-1).*QNondur(1:end-1)+PServ(1:end-1).*QServ(1:end-1)+PG(1:end-1).*QG(1:end-1));
SCons   =   (PNondur(2:end).*QNondur(2:end)+PServ(2:end).*QServ(2:end)+PG(2:end).*QG(2:end))...
    ./(PNondur(1:end-1).*QNondur(2:end)+PServ(1:end-1).*QServ(2:end)+PG(1:end-1).*QG(2:end));
PCons   =   (LCons.*SCons).^.5;    % Fisher price index for private cons.
PCons   =   cumprod(PCons);
PCons   =   PCons/mean(PCons(end-25:end-22));
Cons    =   (NondurN+ServN)./PCons;
QGDPI   =   table7_1(27,:);
PGDPI   =   table7_1(28,:);
LInv    =   (PDur(2:end).*QDur(1:end-1)+PGDPI(2:end).*QGDPI(1:end-1))./(PDur(1:end-1).*QDur(1:end-1)+PGDPI(1:end-1).*QGDPI(1:end-1));
SInv    =   (PDur(2:end).*QDur(2:end)+PGDPI(2:end).*QGDPI(2:end))./(PDur(1:end-1).*QDur(2:end)+PGDPI(1:end-1).*QGDPI(2:end));
PInv    =   (LInv.*SInv).^.5;    % Fisher price index for private inv.
PInv    =   cumprod(PInv);
PInv    =   PInv/mean(PInv(end-25:end-22));
Inv     =   (DurN+GPDIN)./PInv;

GDP     =   GDP(2:end);
fGDP    =   bpass(log(GDP)',6,32);
fCons   =   bpass(log(Cons)',6,32);
fInv    =   bpass(log(Inv)',6,32);
fPrice  =   bpass(log(PInv./PCons)',6,32);
flinv   =   bpass(log(linv),6,32);
flcons  =   bpass(log(lcons),6,32);
fltot   =   bpass(log(lcons+linv),6,32);
bccorr  =   corr([fGDP fCons fInv flinv flcons fPrice fltot]);
subplot(321),nbershading(fGDP,374,594),title('Real GDP')
subplot(323),nbershading(fCons,374,594),title(sprintf('Consumption; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr(1,2),std(fCons)/std(fGDP)))
subplot(325),nbershading(fInv,374,594),title(sprintf('Investment; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr(1,3),std(fInv)/std(fGDP)))
subplot(322),nbershading(fltot,374,594),title(sprintf('Aggregate Labor; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr(1,7),std(fltot)/std(fGDP)))
subplot(324),nbershading(flcons,374,594),title(sprintf('Labor in the cons. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr(1,5),std(flcons)/std(fGDP)))
subplot(326),nbershading(flinv,374,594),title(sprintf('Labor in the inv. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr(1,4),std(flinv)/std(fGDP)))
% suptitle('Business Cycle Frequency')
% print -dps dataplots
print -dps dataplot1

% figure
% f1GDP    =   bpass(log(GDP)',32,80);
% f1Cons   =   bpass(log(Cons)',32,80);
% f1Inv    =   bpass(log(Inv)',32,80);
% f1Price  =   bpass(log(PInv./PCons)',32,80);
% f1linv   =   bpass(log(linv),32,80);
% f1lcons  =   bpass(log(lcons),32,80);
% f1ltot   =   bpass(log(lcons+linv),32,80);
% bccorr1  =   corr([f1GDP f1Cons f1Inv f1linv f1lcons f1Price f1ltot]);
% subplot(321),nbershading(f1GDP,374,594),title('Real GDP')
% subplot(323),nbershading(f1Cons,374,594),title(sprintf('Consumption; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr1(1,2),std(f1Cons)/std(f1GDP)))
% subplot(325),nbershading(f1Inv,374,594),title(sprintf('Investment; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr1(1,3),std(f1Inv)/std(f1GDP)))
% subplot(322),nbershading(f1ltot,374,594),title(sprintf('Aggregate Labor; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr1(1,7),std(f1ltot)/std(f1GDP)))
% subplot(324),nbershading(f1lcons,374,594),title(sprintf('Labor in the cons. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr1(1,5),std(f1lcons)/std(f1GDP)))
% subplot(326),nbershading(f1linv,374,594),title(sprintf('Labor in the inv. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr1(1,4),std(f1linv)/std(f1GDP)))
% % suptitle('8-20 years')
% % print -dps -append dataplots
% print -dps dataplot2
% 
% figure
% f2GDP    =   bpass(log(GDP)',80,160);
% f2Cons   =   bpass(log(Cons)',80,160);
% f2Inv    =   bpass(log(Inv)',80,160);
% f2Price  =   bpass(log(PInv./PCons)',80,160);
% f2linv   =   bpass(log(linv),80,160);
% f2lcons  =   bpass(log(lcons),80,160);
% f2ltot   =   bpass(log(lcons+linv),80,160);
% bccorr2  =   corr([f2GDP f2Cons f2Inv f2linv f2lcons f2Price f2ltot]);
% subplot(321),nbershading(f2GDP,374,594),title('Real GDP')
% subplot(323),nbershading(f2Cons,374,594),title(sprintf('Consumption; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr2(1,2),std(f2Cons)/std(f2GDP)))
% subplot(325),nbershading(f2Inv,374,594),title(sprintf('Investment; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr2(1,3),std(f2Inv)/std(f2GDP)))
% subplot(322),nbershading(f2ltot,374,594),title(sprintf('Aggregate Labor; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr2(1,7),std(f2ltot)/std(f2GDP)))
% subplot(324),nbershading(f2lcons,374,594),title(sprintf('Labor in the cons. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr2(1,5),std(f2lcons)/std(f2GDP)))
% subplot(326),nbershading(f2linv,374,594),title(sprintf('Labor in the inv. good sector; Corr. = %0.5g, Rel. Vol. = %0.5g',bccorr2(1,4),std(f2linv)/std(f2GDP)))
% % suptitle('20-40 years')
% % print -dps -append dataplots
% print -dps dataplot3    

figure
subplot(221),plot(-4:1:4,xcov(fGDP,fInv,4,'coef'),'k','LineWidth',1.5),axis([-4 4 -1 1]),title('Corr(I_{t-j}, y_{t})','FontName','Times New Roman'),set(gca,'FontName','Times New Roman')
subplot(222),plot(-4:1:4,xcov(fGDP,fCons,4,'coef'),'k','LineWidth',1.5),axis([-4 4 -1 1]),title('Corr(C_{t-j}, y_{t})','FontName','Times New Roman'),set(gca,'FontName','Times New Roman')

subplot(223),plot(-4:1:4,xcov(fGDP,flinv,4,'coef'),'k','LineWidth',1.5),axis([-4 4 -1 1]),title('Corr(L_{ I, t-j}, y_{t})','FontName','Times New Roman'),set(gca,'FontName','Times New Roman')
subplot(224),plot(-4:1:4,xcov(fGDP,flcons,4,'coef'),'k','LineWidth',1.5),axis([-4 4 -1 1]),title('Corr(L_{ C, t-j}, y_{t})','FontName','Times New Roman'),set(gca,'FontName','Times New Roman')

