function [ksipfspecific,ksiph] = findksip(gam,lambdaf,lambdaw,muups,muz,beta,del,eta,b,sigmaL,psiL,V,nu,x,eps,alpha,siga,Spp)

%this takes the coefficients for the firm-specific capital versio of the ACEL model and computes the implied
%value of ksip. It checks - by looking over a grid of admissible values of
%ksip - that the solution is unique
          
fixed=1;%if unity, there is a fixed cost and profits are zero in steady state, if zero, the fixed cost is zero and profits are positive in ss
[muzst,rhot,R,Rnu,wt,kb,h,q,m,pie,yt,c,phi,theta]=steady(lambdaf,lambdaw,muups,muz,beta,del,eta,b,sigmaL,psiL,V,nu,x,eps,alpha,fixed);
           
kksip=.001:.1:.99;
skap1=0.1;
skap2=.1;
skap3=1;

ksip=kksip(1);
[psi0,psi1,kap1,kap2,kap3,gamma,zeta,fval] ...
    = getsolveACEL(skap1,skap2,skap3,alpha,siga,theta,yt,phi,del,rhot,muzst,muups,beta,ksip,Spp);
ff=(gamma-gam);
ix=0;
for ii = 2:length(kksip)
    ksip=kksip(ii);
    [psi0,psi1,kap1,kap2,kap3,gamma,zeta,fval] ...
        = getsolveACEL(skap1,skap2,skap3,alpha,siga,theta,yt,phi,del,rhot,muzst,muups,beta,ksip,Spp);    
    if (gamma-gam)*ff < 0
        ix=ix+1;
        II(ix)=ii;
    end
    ff=gamma-gam;
end

if ix > 1 | ix == 0
    error('fatal (findksip) failed to bracket a solution')
end
    
k1=kksip(II(ix)-1);
k2=kksip(II(ix));
[ff1] = ffindksip(k1,alpha,siga,theta,yt,phi,del,rhot,muzst,muups,beta,Spp,gam);
[ff2] = ffindksip(k2,alpha,siga,theta,yt,phi,del,rhot,muzst,muups,beta,Spp,gam);

if ff1*ff2 > 0
    error('fatal (findksip) failed to bracket a solution')
end

options = optimset('Display','off','TolFun',1e-10,'TolX',1e-10,'maxiter',10000,'MaxFun',10000,'LargeScale','off');
[xx,ff,exitflag]=fzero('ffindksip',[k1 k2],options,alpha,siga,theta,yt,phi,del,rhot,muzst,muups,beta,Spp,gam);   

if exitflag == 0
    disp('(findksip) maximum number of function evaluations reached')
elseif exitflag < 0
    error('(findksip) did not converge to a solution')
end

if isnan(ff) == 1 | abs(imag(ff)) > .1e-0 
    error('fatal (findksip) optimized function value not a sensible number')
end

ksipfspecific=xx;

c=[beta -(1+beta+gam) 1];
rr=roots(c);
ksiph=rr(1);
if ksiph > 1 | ksiph < 0
    ksiph=rr(2);
end
if ksiph > 1 | ksiph < 0 | abs(imag(ksiph)) > .1e-10
    disp('WARNING - no value of ksip inside unit circle is consistent with value of gamma used')
end