clear all;
close all;

nomdata     =   xlsread('section1ALL_xls.xls','101 Qtr');
realdata    =   xlsread('section1ALL_xls.xls','102 Qtr');
dridata     =   csvread('DRIdata.csv',1,1);
LBOUTU      =   csvread('LBOUTU.csv',1,1);
load p_i

YRZ         =   1947:.25:2001.75;
GDP         =   nomdata(10,8:227)';
GDPQ        =   realdata(10,4:223)';
GCD         =   nomdata(12,8:227)';
GCN         =   nomdata(13,8:227)';
GCS         =   nomdata(14,8:227)';
GPI         =   nomdata(15,8:227)';
GGE         =   nomdata(29,8:227)';
LBMNU       =   dridata(1:220,1);
LBCPU       =   dridata(1:220,2);
LBOUTU      =   dridata(1:220,3);
FM2         =   dridata(1:220,4);
FYFF        =   dridata(1:220,5);
P16         =   dridata(1:220,6);
IPXMCA      =   dridata(1:220,7);
LBOUTU      =   LBOUTU(1:220);

percaphours =   LBMNU./P16;                 %per capita hours Non-farm Business
pgdp        =   GDP./GDPQ;                  %GDP deflator
ymm         =   (GDP./pgdp)./P16;           %real chain weighted level of output.
cmm         =   ((GCN+GCS+GGE)./pgdp)./P16; %consumption on nondurables and services and government. 
imm         =   ((GCD + GPI)./pgdp)./P16 ;  %consumption of durables and private investment. 
cmmratio    =   (GCN+GCS+GGE)./GDP;
immratio    =   (GCD + GPI)./GDP;
wagez       =   exp(log(LBCPU)-log(pgdp));
capu        =   IPXMCA;                     %capacity utilization

load mzm

chngz = [diff(log(p_i)+log(consdef)-log(pgdp)) diff(log(ymm./percaphours)) diff(log(pgdp))];  
levlz = [log(capu)  log(percaphours) log(ymm./percaphours)-log(wagez) log(cmmratio) log(immratio) FYFF/100 log(GDP./FM2)];
vardata = [chngz levlz(2:end,:)];
vardata =   vardata(49:end,:);
load mzm
% Note: removing the spike in the MZM growth rate makes the VAR
% non-stationary
gr=diff(log(MZMsplice));
gr(96)=[gr(96) (gr(97)+gr(95))/2]*[.5 .5]';
MZMsplice1=exp(log(MZMsplice(1))+[0; cumsum(gr)]);
vardata(:,end)  =   log(GDP(50:end)./MZMsplice1(2:172));
vardata(:,end)  =   log(GDP(50:end)./MZMsplice(2:172));

%   STEP 2: Estimate VARs

nsteps = 30;
nlags = 4;
hascon = 1;
nvars = size(vardata,2);

%   identify a price of investment shock, a tech shock and a fed funds shock. (Fed Funds is the 9th variable in the VAR.)
ffshk = 9;
[impzout,vdout,simdataout,erzout,azeroout,a0betazout,var_explode,V,D] = mkimplrnew(vardata,nlags,nsteps,hascon,[],[],ffshk,'sortlp',0);

yr=1854:1:2002;
for i=1:length(yr),year(1+4*(i-1):4*i,1)=yr(i)*ones(4,1);month(1,1+4*(i-1):4*i)=1:3:10;end
month=month';
dates=datenum(year,month,ones(size(month)));
through =   zeros(size(date));
through(4)      =   1;
through(20)     =   1;
through(31)     =   1;
through(53)     =   1;
through(68)     =   1;
through(101)    =   1;
through(126)    =   1;
through(137)    =   1;
through(150)    =   1;
through(162)    =   1;
through(174)    =   1;
through(188)    =   1;
through(203)    =   1;
through(218)    =   1;
through(236)    =   1;
through(244)    =   1;
through(261)    =   1;
through(271)    =   1;
through(283)    =   1;
through(296)    =   1;
through(317)    =   1;
through(338)    =   1;
through(368)    =   1;
through(384)    =   1;
through(402)    =   1;
through(418)    =   1;
through(429)    =   1;
through(468)    =   1;
through(485)    =   1;
through(507)    =   1;
through(516)    =   1;
through(549)    =   1;


peak            =   zeros(size(date));
peak(14)        =   1;
peak(27)        =   1;
peak(45)        =   1;
peak(62)        =   1;
peak(79)        =   1;
peak(113)       =   1;
peak(134)       =   1;
peak(147)       =   1;
peak(157)       =   1;
peak(168)       =   1;
peak(183)       =   1;
peak(196)       =   1;
peak(214)       =   1;
peak(225)       =   1;
peak(237)       =   1;
peak(259)       =   1;
peak(265)       =   1;
peak(278)       =   1;
peak(291)       =   1;
peak(303)       =   1;
peak(334)       =   1;
peak(365)       =   1;
peak(380)       =   1;
peak(398)       =   1;
peak(415)       =   1;
peak(426)       =   1;
peak(464)       =   1;
peak(480)       =   1;
peak(505)       =   1;
peak(511)       =   1;
peak(547)       =   1;
peak(589)       =   1;

initvar =   592-size(simdataout,1)+1;
endvar  =   592;
[i,j,s]=find(through);
j  =  [j, length(dates)];
[ii,jj,ss]=find(peak);
varnamz = char('Output','MZM Growth','Inflation','Fed Funds','Capacity Util','Avg Hours','Real Wage','Consumption','Investment','Velocity','Price of Inv.');

figure, orient landscape
for zx=1:11
    subplot(4,3,zx)
    var1    =   100*simdataout(:,zx,5);
    var2    =   100*simdataout(:,zx,1);  
    if zx==4
        var1    =   simdataout(:,zx,5);
        var2    =   simdataout(:,zx,1);  
    end
    maxvar  =   1.05*max(max(var1),max(var2));
    minvar1=min(var1)-.05*abs(min(var1));
    minvar2=min(var2)-.05*abs(min(var2));
    minvar  =   min(minvar1,minvar2);
    hold on;
    for ll=1:length(jj)
        area([dates(jj(ll)) dates(j(ll+1))],[1.01 1.01]*maxvar,minvar,'FaceColor',[ 0.75 0.75 0.75 ])
    end
    axis([dates(initvar) dates(endvar) minvar maxvar])
    plot(dates(initvar:endvar),var1,'k','LineWidth',1.5)
    plot(dates(initvar:endvar),var2,'k')
    hold off
    set(gca,'XtickLabel',datestr(dates(initvar:40:endvar),17))
    set(gca,'Xtick',dates(initvar:40:endvar))
    set(gca,'FontSize',9)
    set(gca,'FontName','Times New Roman')
    title(varnamz(zx,:));
end
suptitle('Figure 8: Historical decomposition - embodied technology shocks only')
print -dps2 figure8

figure, orient landscape
for zx=1:11
    subplot(4,3,zx)
    var1    =   100*simdataout(:,zx,5);
    var2    =   100*simdataout(:,zx,2);  
    if zx==4
        var1    =   simdataout(:,zx,5);
        var2    =   simdataout(:,zx,2);  
    end
    maxvar  =   1.05*max(max(var1),max(var2));
    minvar1=min(var1)-.05*abs(min(var1));
    minvar2=min(var2)-.05*abs(min(var2));
    minvar  =   min(minvar1,minvar2);
    hold on;
    for ll=1:length(jj)
        area([dates(jj(ll)) dates(j(ll+1))],[1.01 1.01]*maxvar,minvar,'FaceColor',[ 0.75 0.75 0.75 ])
    end
    axis([dates(initvar) dates(endvar) minvar maxvar])
    plot(dates(initvar:endvar),var1,'k','LineWidth',1.5)
    plot(dates(initvar:endvar),var2,'k')
    hold off
    set(gca,'XtickLabel',datestr(dates(initvar:40:endvar),17))
    set(gca,'Xtick',dates(initvar:40:endvar))
    set(gca,'FontSize',9)
    set(gca,'FontName','Times New Roman')
    title(varnamz(zx,:));
end
suptitle('Figure 7: Historical decomposition - neutral technology shocks only')
print -dps2 figure7

figure, orient landscape
for zx=1:11
    subplot(4,3,zx)
    var1    =   100*simdataout(:,zx,5);
    var2    =   100*simdataout(:,zx,3);  
    if zx==4
        var1    =   simdataout(:,zx,5);
        var2    =   simdataout(:,zx,3);  
    end
    maxvar  =   1.05*max(max(var1),max(var2));
    minvar1=min(var1)-.05*abs(min(var1));
    minvar2=min(var2)-.05*abs(min(var2));
    minvar  =   min(minvar1,minvar2);
    hold on;
    for ll=1:length(jj)
        area([dates(jj(ll)) dates(j(ll+1))],[1.01 1.01]*maxvar,minvar,'FaceColor',[ 0.75 0.75 0.75 ])
    end
    axis([dates(initvar) dates(endvar) minvar maxvar])
    plot(dates(initvar:endvar),var1,'k','LineWidth',1.5)
    plot(dates(initvar:endvar),var2,'k')
    hold off
    set(gca,'XtickLabel',datestr(dates(initvar:40:endvar),17))
    set(gca,'Xtick',dates(initvar:40:endvar))
    set(gca,'FontSize',9)
    set(gca,'FontName','Times New Roman')
    title(varnamz(zx,:));
end
suptitle('Figure 6: Historical decomposition - monetary policy shocks only')
print -dps2 figure6

figure, orient landscape
for zx=1:11
    subplot(4,3,zx)
    var1    =   100*simdataout(:,zx,5);
    var2    =   100*simdataout(:,zx,4);  
    if zx==4
        var1    =   simdataout(:,zx,5);
        var2    =   simdataout(:,zx,4);  
    end
    maxvar  =   1.05*max(max(var1),max(var2));
    minvar1=min(var1)-.05*abs(min(var1));
    minvar2=min(var2)-.05*abs(min(var2));
    minvar  =   min(minvar1,minvar2);
    hold on;
    for ll=1:length(jj)
        area([dates(jj(ll)) dates(j(ll+1))],[1.01 1.01]*maxvar,minvar,'FaceColor',[ 0.75 0.75 0.75 ])
    end
    axis([dates(initvar) dates(endvar) minvar maxvar])
    plot(dates(initvar:endvar),var1,'k','LineWidth',1.5)
    plot(dates(initvar:endvar),var2,'k')
    hold off
    set(gca,'XtickLabel',datestr(dates(initvar:40:endvar),17))
    set(gca,'Xtick',dates(initvar:40:endvar))
    set(gca,'FontSize',9)
    set(gca,'FontName','Times New Roman')
    title(varnamz(zx,:));
end
suptitle('Figure 9: Historical decomposition - monetary policy and technology shocks')
print -dps2 figure9