clear all
% close all

setpath

% (1-L)*p_I, (1-L)*APL, (1-L)*GDP_def, Capacity, Hours, APL-wage, C/Y, I/Y,
% FYFF, MZM velocity
% 1959:Q2-2001:Q4
load data

% Demean and rearrange: remove utilization, and put (1-L)*p_I last
data    =   data-repmat(mean(data),size(data,1),1);
data    =   data(:,[2 3 5:10 1])';

getparams_s
setup
%load up the parameters
do

taux=[1 2 4 5 8 10 14 16];%equations with limited information

% kalman_matrices has as optional 4th argument the number of iterations on the Riccati equation for initV;
% if no 4th argument is specified, it uses dare.m to solve the Riccati equation
% [H,F,Q,initV,alf,tau,tau_bar,tau_theta,VV,ee]    =   kalman_matrices(xx,par,taux,20);

xx  =   [-0.10759557545844
   0.53292117442133
   4.34876364146126
   0.89768969555496
  -0.27206953683268
  -0.50727441048085
   0.27934952698222
   0.05230212542246
   0.19827027114065
   1.11686672046857
   1.46181794612030
   0.67432286843049
   0.62482616128916
   0.94443213566974
   0.53459153554651
   0.36244970768100];
initR       =   .005*ones(9);
initR       =   .00000001*eye(9);
initR(1,1)  =   .000000001;
initR(2,2)  =   .0000000001;
initR(3,3)  =   .0000000001;
initR(4,4)  =   .0000000001;
initR(9,9)  =   .00000000001;


optimize = 0; % if set to 1 does the estimation

warning off MATLAB:eigs:TooManyRequestedEigsForComplexNonsym
if optimize==1
    init    =   [imap(xx),log(diag(initR))'];
    options = optimset('Display','iter','TolFun',1e-8,'TolX',1e-8,'maxiter',50000,'MaxFun',50000,'LargeScale','off');
    [DD,ValDD,exitflag]=fminsearch('ssloglik',init,options,par,taux,data,10);
    
    if isnan(ValDD) == 1 | abs(imag(ValDD)) > .1e-0 
        error('fatal (main_kalman) optimized function value not a sensible number')
    end
    
    xx      =   map(DD(1:end-9))
    RR      =   exp(DD(end-8:end));
    initx   =   zeros(42,1);
    RRmle   =   diag(RR);
    
    [Hmle,Fmle,Qmle,initV]=kalman_matrices(xx,par,taux);
%     [xsmooth,Vsmooth,VVsmooth,loglik]=kalman_smoother(data,Fmle,Hmle,Qmle,RRmle,initx,initV);
%     datasmooth=Hmle*xsmooth;
    bigvar  =   zeros(51);
    bigvar(1:42,1:42)   =   Qmle;
    bigvar(43:51,43:51) =   RRmle;
    [logl,pred,vpred] = kalcvf(data, 0, zeros(42,1), Fmle, zeros(9,1), Hmle, bigvar);
    [xsmooth,Vsmooth1] = kalcvs(data, zeros(42,1), Fmle, zeros(9,1), Hmle, bigvar, pred, vpred);
    datasmooth=Hmle*xsmooth;
    save mle_est xx RRmle Hmle Fmle Qmle initV datasmooth
end

if optimize==0
    load mle_est
    initx   =   zeros(42,1);
    [Hmle,Fmle,Qmle,initV]=kalman_matrices(xx,par,taux);
    [xsmooth,Vsmooth,VVsmooth,loglik]=kalman_smoother(data,Fmle,Hmle,Qmle,initR,initx,initV);
    bigvar  =   zeros(51);
    bigvar(1:42,1:42)   =   Qmle;
    bigvar(43:51,43:51) =   initR;
    [logl,pred,vpred] = kalcvf(data, 0, zeros(42,1), Fmle, zeros(9,1), Hmle, bigvar);
    [xsmooth1,Vsmooth1] = kalcvs(data, zeros(42,1), Fmle, zeros(9,1), Hmle, bigvar, pred, vpred);
    max(max(xsmooth-xsmooth1))
    datasmooth=Hmle*xsmooth1;
end

figure
labels  =   char('(1-L)*APL', '(1-L)*GDP def', 'Hours', 'APL-wage', 'C/Y', 'I/Y', 'FYFF', 'MZM velocity','(1-L)*p_I');
for i = 1:9,
    subplot(3,3,i)
    plot(1959.25:.25:2001.75,data(i,:),1959.25:.25:2001.75,datasmooth(i,:))
    title(labels(i,:));
    axis tight
end
suptitle('Data (blue) vs. Kalman smoothed est. (green)')
% print -dps smooth.ps