%   Riccardo DiCecio
%   08/21/2003

%   STEP 1: Read in the DATA
clear all;
close all;

nomdata     =   xlsread('c:\data\nipa\section1ALL_xls.xls','101 Qtr');
realdata    =   xlsread('c:\data\nipa\section1ALL_xls.xls','102 Qtr');
dridata     =   csvread('DRIdata.csv',1,1);
load p_i

YRZ         =   1947:.25:2001.75;
GDP         =   nomdata(10,8:227)';
GDPQ        =   realdata(10,4:223)';
GCD         =   nomdata(12,8:227)';
GCN         =   nomdata(13,8:227)';
GCS         =   nomdata(14,8:227)';
GPI         =   nomdata(15,8:227)';
GGE         =   nomdata(29,8:227)';
LBMNU       =   dridata(1:220,1);
LBCPU       =   dridata(1:220,2);
LBOUTU      =   dridata(1:220,3);
FM2         =   dridata(1:220,4);
FYFF        =   dridata(1:220,5);
P16         =   dridata(1:220,6);
IPXMCA      =   dridata(1:220,7);

percaphours =   LBMNU./P16;                 %per capita hours Non-farm Business
pgdp        =   GDP./GDPQ;                  %GDP deflator
ymm         =   (GDP./pgdp)./P16;           %real chain weighted level of output.
cmm         =   ((GCN+GCS+GGE)./pgdp)./P16; %consumption on nondurables and services and government. 
imm         =   ((GCD + GPI)./pgdp)./P16 ;  %consumption of durables and private investment. 
cmmratio    =   (GCN+GCS+GGE)./GDP;
immratio    =   (GCD + GPI)./GDP;
wagez       =   exp(log(LBCPU)-log(pgdp));
capu        =   IPXMCA;                     %capacity utilization

chngz = [diff(log(p_e)) diff(log(ymm./percaphours)) diff(log(pgdp))];  
levlz = [log(capu) log(percaphours) log(ymm./percaphours)-log(wagez) log(cmmratio) log(immratio) FYFF/100 log(GDP./FM2)];
levlz = [log(capu) log(percaphours) log(ymm./percaphours)-log(wagez) log(cmmratio) log(immratio) FYFF/100];
vardata = [chngz levlz(2:end,:)];
vardata =   vardata(49:end,:);

%   STEP 2: Estimate VARs

nsteps  = 20;
nlags   = 4;
hascon  = 1;
nvars   = size(vardata,2);

%   identify a price of investment shock, a tech shock and a fed funds shock. (Fed Funds is the 9th variable in the VAR.)
ffshk = 9;
[impzout,vdout,simdataout,erzout,azeroout,a0betazout,var_explode,V,D] = mkimplrnew(vardata,nlags,nsteps,hascon,[],[],ffshk,'sortlp_s');
 
%   STEP 3: Make Bootstrap confidence intervals. 
 
ndraws = 1000;
warning off
% [simimp,simvd,count_var_expl] = mkimplrnewci(vardata,nlags,nsteps,hascon,1,10,ffshk,'sortlp',ndraws);
[simimp,simvd,count_var_expl] = mkimplrnewci(vardata,nlags,nsteps,hascon,[],[],ffshk,'sortlp_s',ndraws);
warning on
varnamz = char('Output','Inflation','Fed Funds','Capacity Util','Avg Hours','Real Wage','Consumption','Investment','Price of Inv.');
% [impzout,vdout,simdataout,erzout,azeroout,a0betazout,var_explode,V,D] = mkimplrnew(vardata,nlags,nsteps,hascon,[],[],ffshk,'sortlp',0);
pnstep=20; 
figure, orient landscape
for zx=1:9
    subplot(3,3,zx)
    si = squeeze(simimp(1:pnstep,zx,1,:));
    stdirf1(:,zx)    =   std(si,1,2);
    plot(0:pnstep-1,-100*impzout(1:pnstep,zx,1),'k-')
    hold on
    grpyat = [(0:pnstep-1)' 100*(-impzout(1:pnstep,zx,1)-1.96*std(si,1,2)) ; (pnstep-1:-1:0)' 100*(-impzout(pnstep:-1:1,zx,1)+1.96*std(si(pnstep:-1:1,:),1,2))];
    patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);     
    plot(0:pnstep-1,-100*impzout(1:pnstep,zx,1),'k-')
    title(varnamz(zx,:));
%     xlabel('Periods After Shock')
    axis tight
end
suptitle('Embodied Technology Shock'),print -dps2 figirfpi

figure, orient landscape
for zx=1:9
    subplot(3,3,zx)
    si = squeeze(simimp(1:pnstep,zx,2,:));
    stdirf2(:,zx)    =   std(si,1,2);
    plot(0:pnstep-1,100*impzout(1:pnstep,zx,2),'k-')
    hold on
    grpyat = [(0:pnstep-1)' 100*(impzout(1:pnstep,zx,2)-1.96*std(si,1,2)) ; (pnstep-1:-1:0)' 100*(impzout(pnstep:-1:1,zx,2)+1.96*std(si(pnstep:-1:1,:),1,2))];
    patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);     
    plot(0:pnstep-1,100*impzout(1:pnstep,zx,2),'k-')
    title(varnamz(zx,:));
%     xlabel('Periods After Shock')
    axis tight
end
suptitle('Neutral Technology Shock'),print -dps2 -append figirfpi

figure, orient landscape
for zx=1:9
    subplot(3,3,zx)
    si = squeeze(simimp(1:pnstep,zx,3,:));
    stdirf3(:,zx)    =   std(si,1,2);
    plot(0:pnstep-1,-100*impzout(1:pnstep,zx,3),'k-')
    hold on
    grpyat = [(0:pnstep-1)' 100*(-impzout(1:pnstep,zx,3)-1.96*std(si,1,2)) ; (pnstep-1:-1:0)' 100*(-impzout(pnstep:-1:1,zx,3)+1.96*std(si(pnstep:-1:1,:),1,2))];
    patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);     
    plot(0:pnstep-1,-100*impzout(1:pnstep,zx,3),'k-')
    title(varnamz(zx,:));
%     xlabel('Periods After Shock')
    axis tight
end
suptitle('Fed Funds Shock'),print -dps2 -append figirfpi

% The minuses reflect the fact that a monetary shock in the model is an
% increase in the money growth rate (hence a decrease in the interest rate)
% and that an embodied tech shock reduces the relative price of investment
% goods
impzout(:,:,1)  =   -impzout(:,:,1);
impzout(:,:,3)  =   -impzout(:,:,3);
impzvar =   impzout(1:pnstep,:,1:3);
dimpzvar    =   diff(impzvar);
impzvar =   reshape(impzvar,1,[]);
dimpzvar    =   reshape(dimpzvar,1,[]);
tt      =   2*pnstep*size(varnamz,1);
mprestr =   [1:tt,find(abs(impzvar(tt+1:end))>1e-8)+tt];
impzvar =   impzvar(mprestr);
stdirf  =   [reshape(stdirf1,[],1);reshape(stdirf2,[],1);reshape(stdirf3,[],1)];
stdirf  =   stdirf(mprestr);
Vmat    =   sparse(diag(stdirf));
save var20 impzvar dimpzvar Vmat
save compare20 simimp impzout pnstep
