function [xx,bb] = timedecomp(a0betazout,azeroout,erzout,T,sortlev,rnd,cnst,imp,plt,pl,pu,Il,Iu,nlags)

%T:        T > number of observations in data, draw shocks by randomly drawing, with replacement, from the 
%          fitted residuals, from the VAR system (in this case, rnd is set
%          to unity)
%          if T == number of observations in data (167), look to rnd to decide how to
%          sample shocks
%
%cnst:     if 0, set the constant in VAR to zero.

%rnd:      if unity, draw the disturbances by bootstrapping the fitted residuals, if zero, draw actual sequence of fitted disturbances
%          if 3 draw from Normal distribution, if 4, bootstrap from three
%          distinct episodes: before Il, Il to Iu, after Iu; if 5, do as in
%          4 but always draw the shocks between Il and Iu in the same
%          sequence as in the actual data.

%imp:      if unity, compute impulse responses to shocks (T, cnst, rnd are overriden in this case.)

%pl, pu    lower and upper periods, respectively, for components extracted by band pass filter

%monte:    if unity, generate data using normal random number generator.

if imp == 1
    T=20;
    rnd=0;
    cnst=0;
end

TT=size(erzout,1);

if T > TT
    if rnd == 0;error('fatal (timedecomp) wrong setting for rnd');end
end

const=a0betazout(:,1);
if cnst == 0;const=zeros(10,1);end

nn=size(a0betazout,1);
AA=zeros(nn,nn);
for ii = 1:nlags
    A(:,:,ii)=a0betazout(:,[(ii-1)*nn+2:ii*nn+1]);
    AA=AA+A(:,:,ii);
end
%A(:,:,1)=a0betazout(:,[2:nn+1]);
%A(:,:,2)=a0betazout(:,[nn+2:2*nn+1]);
%A(:,:,3)=a0betazout(:,[2*nn+2:3*nn+1]);
%A(:,:,4)=a0betazout(:,[3*nn+2:4*nn+1]);
C=inv(azeroout);

%system is:
%y(t) = const + A1*y(t-1) + A2*y(t-2) + A3*y(t-3) + A4*y(t-4) + C*erzout(t)
mY=(eye(10)-AA)\const;

I1=zeros(10,10);
I2=zeros(10,10);
I9=zeros(10,10);
Iall=zeros(10,10);
I1(1,1)=1;
I2(2,2)=1;
I9(9,9)=1;
tt=[1 2 9];

if imp == 1 
    [V] = getV(erzout);
    erzout(:,tt)=zeros(TT,length(tt));
    erzout(3,tt)=sqrt(diag(V(tt,tt)))';
end

for jj = 1:3
    Iall(tt(jj),tt(jj))=1;
end

%rand('state',0);
rr=rand(1,T);
II=floor(rr*TT);
I=find(II==0);
II(I)=ones(size(I));
if rnd==0;II=1:T;end

if rnd >= 4
    rr=rand(1,Il-1);
    II1=round(1+rr*(Il-2));
    if rnd == 5
        II2=Il:Iu;
    else
        rr=rand(1,Iu-Il+1);
        II2=round(Il+rr*(Iu-Il));
    end
    rr=rand(1,T-Iu);
    II3=round(Iu+1+rr*(T-Iu-1));
    II=[II1 II2 II3];
end

if rnd == 3
    [V] = getV(erzout);    
    DD=diag(sqrt(diag(V)));
    C=inv(azeroout)*DD;
    erzout=randn(T,10);
    II=1:T;
end

for ii = 1:T
    Y(:,ii)=const+C*erzout(II(ii),:)';
    Y1(:,ii)=const+C*I1*erzout(II(ii),:)';
    Y2(:,ii)=const+C*I2*erzout(II(ii),:)';
    Y9(:,ii)=const+C*I9*erzout(II(ii),:)';
    Yall(:,ii)=const+C*Iall*erzout(II(ii),:)';
    if ii < 5
        for jj = ii:nlags
            Y(:,ii)=Y(:,ii)+A(:,:,jj)*mY;
            Y1(:,ii)=Y1(:,ii)+A(:,:,jj)*mY;
            Y2(:,ii)=Y2(:,ii)+A(:,:,jj)*mY;
            Y9(:,ii)=Y9(:,ii)+A(:,:,jj)*mY;
            Yall(:,ii)=Yall(:,ii)+A(:,:,jj)*mY;
            end
    end
    for jj = 1:min(nlags,ii-1)
        Y(:,ii)=Y(:,ii)+A(:,:,jj)*Y(:,ii-jj);
        Y1(:,ii)=Y1(:,ii)+A(:,:,jj)*Y1(:,ii-jj);
        Y2(:,ii)=Y2(:,ii)+A(:,:,jj)*Y2(:,ii-jj);
        Y9(:,ii)=Y9(:,ii)+A(:,:,jj)*Y9(:,ii-jj);
        Yall(:,ii)=Yall(:,ii)+A(:,:,jj)*Yall(:,ii-jj);
    end
end

if T == TT & max(abs(mY)) < .1e-10 & rnd == 0
    [rY,rY1,rY2,rY9,simdataout] = robsimulate(azeroout,a0betazout,erzout,sortlev);
    if max(max(abs(rY-Y))) > .1e-10 | max(max(abs(rY1-Y1))) > .1e-10 | max(max(abs(rY2-Y2))) > .1e-10 ...
            | max(max(abs(rY9-Y9))) > .1e-10 
        error('fatal (timedecomp) raw simulated data has problem')
    end
end

[YY,titz] = tran(Y);
[YY1,titz] = tran(Y1);
[YY2,titz] = tran(Y2);
[YY9,titz] = tran(Y9);
[YYall,titz] = tran(Yall);

if T >= TT
    
    tt=1:T;
    [d,t]=hpfast(YY(1,tt)',1600);
    [d1,t]=hpfast(YY1(1,tt)',1600);
    [d2,t]=hpfast(YY2(1,tt)',1600);
    [d9,t]=hpfast(YY9(1,tt)',1600);
    [dall,t]=hpfast(YYall(1,tt)',1600);
    if plt == 1
        subplot(221)
        plot(tt,d,tt,d1)
        title('disembodied')
        subplot(222)
        plot(tt,d,tt,d2)
        title('neutral')
        subplot(223)
        plot(tt,d,tt,d9)
        title('policy')
        subplot(224)
    end
    
    s1=var(d1)/var(d);
    s2=var(d2)/var(d);
    s9=var(d9)/var(d);
    sall=var(dall)/var(d);
    xx=[s1 s2 s9 sall];
    
    if T <= 4000
        
        [b] = bpassm(YY(1,tt)',pl,pu);
        [b1] = bpassm(YY1(1,tt)',pl,pu);
        [b2] = bpassm(YY2(1,tt)',pl,pu);
        [b9] = bpassm(YY9(1,tt)',pl,pu);
        [ball] = bpassm(YYall(1,tt)',pl,pu);
        
        s1=var(b1)/var(b);
        s2=var(b2)/var(b);
        s9=var(b9)/var(b);
        sall=var(ball)/var(b);
        bb=[s1 s2 s9 sall];
        
        iplt=0;
        if iplt == 1
            subplot(221)
            plot(II,YY(1,:),II,YY9(1,:),II,YY9(1,:),'*')
            title('money shocks only')
            subplot(222)
            plot(II,YY(1,:),II,YY1(1,:),II,YY1(1,:),'*')
            title('disembodied shocks only')
            subplot(223)
            plot(II,YY(1,:),II,YY2(1,:),II,YY2(1,:),'*')
            title('neutral shocks only')
            subplot(224)
            plot(II,YY(1,:),II,YYall(1,:),II,YYall(1,:),'*')
            title('all three shocks')
        end
        
    else
        
        bb=[];
        
    end
    
else    
    
    subplot(221)
    plot(-100*YY9(1,:))
    title('Response of Output to Money Shocks')
    subplot(222)
    plot(100*YY1(1,:))
    title('Response of Output to Disembodied shocks')
    subplot(223)
    plot(100*YY2(1,:))
    title('Response of Output to Neutral shocks')
    
    xx=[];
    bb=[];
    
end